import os
import pandas as pd
import numpy as np
import scipy.stats as stats
import statsmodels.api as sm

def read_stock_data(directory):
    stock_data = []
    for filename in os.listdir(directory):
        if filename.endswith('_5min_normalized_indicators.csv'):
            filepath = os.path.join(directory, filename)
            df = pd.read_csv(filepath, parse_dates=['timestamp'])
            stock_data.append(df)

    # Concatenate all the DataFrames into a single DataFrame
    combined_df = pd.concat(stock_data)

    return [combined_df]

def calculate_t_test(data, indicator):
    combined_df = data[0]
    t_val, _ = stats.ttest_1samp(combined_df[indicator].dropna(), 0)
    return [t_val]

def calculate_f_test(data, indicators):
    combined_df = data[0]
    # Check if all indicators are present in the DataFrame
    if all(indicator in combined_df.columns for indicator in indicators):
        # Check if all indicators have at least one valid value
        if all(not combined_df[indicator].dropna().empty for indicator in indicators):
            F, _ = stats.f_oneway(*(combined_df[indicator].dropna() for indicator in indicators))
            return [F]
        else:
            print("Skipping F-test for DataFrame with missing values")
            return [None]
    else:
        print("Skipping F-test for DataFrame with missing indicators")
        return [None]

def calculate_information_ratio(data, indicator):
    combined_df = data[0]
    excess_returns = combined_df[indicator].dropna() - combined_df['close'].pct_change().dropna()
    information_ratio = excess_returns.mean() / excess_returns.std()
    return [information_ratio]

def calculate_beta_and_jensens_alpha(data, target_column):
    betas = []
    jensens_alphas = []
    for df in data:
        # Check if the target_column exists and has at least two valid values
        if target_column in df.columns and len(df[target_column].dropna()) >= 2:
            X = df[target_column].pct_change().dropna()
            
            # Check if the 'market_returns' column exists
            if 'market_returns' in df.columns:
                y = df['market_returns'].pct_change().dropna()
            else:
                y = None
                print(f"Skipping calculation for DataFrame with missing 'market_returns' column: {df}")
                
            X = sm.add_constant(X)

            # Check if X and y have the same length
            if y is not None and len(X) == len(y):
                model = sm.OLS(y, X).fit()
                betas.append(model.params[target_column])
                jensens_alphas.append(model.params['const'])
            else:
                print(f"Skipping calculation for DataFrame with mismatched lengths: {df}")
                betas.append(None)
                jensens_alphas.append(None)
        else:
            print(f"Skipping calculation for DataFrame with missing or insufficient values in target_column: {df}")
            betas.append(None)
            jensens_alphas.append(None)
    return betas, jensens_alphas

stock_data = read_stock_data('stockDataIndiSmall')
indicators = [ 'SMA_10',  'EMA_10']
t_test_results = calculate_t_test(stock_data, 'high')
f_test_results = calculate_f_test(stock_data, indicators)
information_ratio_results = calculate_information_ratio(stock_data, 'volume')
beta_results, jensens_alpha_results = calculate_beta_and_jensens_alpha(stock_data, 'close')

print("T-test results:", sum(t_test_results))
print("F-test results:", sum(f_test_results))
print("Information Ratio results:", sum(information_ratio_results))
beta_sum = sum(val for val in beta_results if val is not None)
print("Beta results:", beta_sum)
jensens_alpha_sum = sum(val for val in jensens_alpha_results if val is not None)
print("Jensen's Alpha results:", jensens_alpha_sum)
