import os
import pandas as pd
import pandas_ta as ta
import numpy as np


class TechnicalIndicators:
    def __init__(self, data):
        self.data = data

    def SMA(self, period, column_name):
        return self.data[column_name].rolling(window=period).mean()

    def EMA(self, period, column_name):
        return self.data[column_name].ewm(span=period, adjust=False).mean()

    def MACD(self, short_period, long_period, signal_period, column_name):
        short_EMA = self.EMA(short_period, column_name)
        long_EMA = self.EMA(long_period, column_name)
        self.data['MACD'] = short_EMA - long_EMA
        self.data['Signal'] = self.EMA(signal_period, 'MACD')
        return self.data

    def RSI(self, period, column_name):
        delta = self.data[column_name].diff()
        gain = delta.where(delta > 0, 0)
        loss = -delta.where(delta < 0, 0)
        avg_gain = gain.rolling(window=period).mean()
        avg_loss = loss.rolling(window=period).mean()
        RS = avg_gain / avg_loss
        self.data['RSI'] = 100 - (100 / (1 + RS))
        return self.data

    def BollingerBands(self, period, column_name):
        sma = self.SMA(period, column_name)
        std_dev = self.data[column_name].rolling(window=period).std()
        self.data['BB_upper'] = sma + (2 * std_dev)
        self.data['BB_lower'] = sma - (2 * std_dev)
        return self.data

    def ATR(self, period):
        high_low = self.data['high'] - self.data['low']
        high_close = np.abs(self.data['high'] - self.data['close'].shift())
        low_close = np.abs(self.data['low'] - self.data['close'].shift())
        TR = pd.concat([high_low, high_close, low_close], axis=1).max(axis=1)
        self.data['ATR'] = TR.rolling(window=period).mean()
        return self.data

    def OBV(self):
        volumes = self.data['volume']
        changes = self.data['close'].diff()
        obv = np.where(changes > 0, volumes, np.where(changes < 0, -volumes, 0)).cumsum()
        self.data['OBV'] = obv
        return self.data

    def StochasticOscillator(self, k_period, d_period):
        high_max = self.data['high'].rolling(window=k_period).max()
        low_min = self.data['low'].rolling(window=k_period).min()
        self.data['%K'] = 100 * ((self.data['close'] - low_min) / (high_max - low_min))
        self.data['%D'] = self.SMA(d_period, '%K')
        return self.data

    def ROC(self, period, column_name):
        self.data['ROC'] = ((self.data[column_name] - self.data[column_name].shift(period)) / self.data[column_name].shift(period)) * 100
        return self.data

    def CCI(self, period):
        TP = (self.data['high'] + self.data['low'] + self.data['close']) / 3
        sma_tp = sma_tp = TP.rolling(window=period).mean()
        mean_deviation = TP.rolling(window=period).apply(lambda x: np.mean(np.abs(x - x.mean())))
        self.data['CCI'] = (TP - sma_tp) / (0.015 * mean_deviation)
        return self.data

class TradingStrategy:
    def __init__(self, data, stop_loss, take_profit):
        self.data = data
        self.stop_loss = stop_loss
        self.take_profit = take_profit           
    def apply_strategy(self):
        balance = 2000
        position = 0
        open_price = 0
        trailing_stop = 0

        for i in range(1, len(self.data)):
            current_close = self.data.at[i, 'close']
            previous_close = self.data.at[i - 1, 'close']

            if current_close > previous_close and position == 0:
                # Check if bullish conditions are met
                if self.is_bullish(i):
                    # Buy
                    position = balance / current_close
                    open_price = current_close
                    balance = 0
            elif position != 0:
                trailing_stop = max(trailing_stop, current_close * (1 - self.stop_loss))
                # Check for stop loss and take profit
                if current_close <= trailing_stop or self.hit_take_profit(current_close, open_price):
                    # Sell
                    balance = position * current_close
                    position = 0

        # Sell remaining position
        if position != 0:
            balance = position * self.data.at[len(self.data) - 1, 'close']
            position = 0

        return balance

    def is_bullish(self, index):
        one_hour_volume_price = sum(self.data.loc[max(0, index - 10):index, 'VolumePrice'])
        conditions = [
            #self.data.at[index, 'RSI'] < 70,
            #self.data.at[index, 'MACD'] > self.data.at[index, 'Signal'],
            #self.data.at[index, '%K'] < 80,
            #self.data.at[index, '%D'] < 80,
            #self.data.at[index, 'CCI'] < 100,
            # self.data.at[index, 'SMA_50'] > self.data.at[index, 'SMA_200'],
            # self.data.at[index, 'close'] > self.data.at[index, 'BBU_20'],
            one_hour_volume_price > 100000,
        ]
        return all(conditions)

    def hit_stop_loss(self, current_close, open_price):
        return current_close <= open_price * (1 - self.stop_loss)

    def hit_take_profit(self, current_close, open_price):
        return current_close >= open_price * (1 + self.take_profit)
def add_indicators(data):
    # Add Volume * Price
    data['VolumePrice'] = data['volume'] * data['close']
    return data

def main():
    output_file = 'backtest_results_withVolume_10000.csv'
    results = []
    stop_loss = 0.02
    take_profit = 0.05

    # Set directory path
    directory = 'stockDataIndiSmall'

    # Loop through files in directory
    for filename in os.listdir(directory):
        if filename.endswith('_5min_normalized_indicators.csv'):
            stock_name = filename.split('_')[0]
            file_path = os.path.join(directory, filename)
            data = pd.read_csv(file_path)
            # Add features to the dataset
            data = add_indicators(data)

            ts = TradingStrategy(data, stop_loss, take_profit)
            final_balance = ts.apply_strategy()
            results.append([stock_name, final_balance])

    results_df = pd.DataFrame(results, columns=['Stock', 'Final Balance'])
    results_df.to_csv(output_file, index=False)

if __name__ == '__main__':
    main()
