import os
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

def load_data(stock_directory):
    data = {}
    for file in os.listdir(stock_directory):
        if file.endswith(".csv"):
            file_path = os.path.join(stock_directory, file)
            stock_name = file.split("_")[0]
            data[stock_name] = pd.read_csv(file_path)
    return data
class Trade:
    def __init__(self, stock, entry_timestamp, entry_price):
        self.stock = stock
        self.entry_timestamp = entry_timestamp
        self.entry_price = entry_price
        self.exit_timestamp = None
        self.exit_price = None

    def exit(self, exit_timestamp, exit_price):
        self.exit_timestamp = exit_timestamp
        self.exit_price = exit_price

    @property
    def profit(self):
        if self.exit_price is None:
            return np.nan
        return self.exit_price - self.entry_price

    @property
    def duration(self):
        if self.exit_timestamp is None:
            return np.nan
        return self.exit_timestamp - self.entry_timestamp


class Backtest:
    def __init__(self, stock_data, min_volume_price=0, profit_taker_multiplier=0.06,
                 stop_loss_multiplier=0.03, initial_balance=1000):
        self.stock_data = stock_data
        self.min_volume_price = min_volume_price
        self.profit_taker_multiplier = profit_taker_multiplier
        self.stop_loss_multiplier = stop_loss_multiplier
        self.initial_balance = initial_balance
        self.trades = []
        self.trade_data = {}
        self.max_drawdown = 0
        self.total_balance = len(stock_data) * initial_balance
        self.highest_balance = self.total_balance

    def run(self):
        for stock, data in self.stock_data.items():
            self.trade_data[stock] = {
                'total_profit': 0,
                'total_trades': 0,
                'winning_trades': 0,
                'losing_trades': 0,
                'balance': self.initial_balance
            }

            position = None
            close_prices = data['close'].values
            volumes = data['volume'].values
            highs = data['high'].values

            for i in range(1, len(close_prices)):
                if position:
                    profit = close_prices[i] - position.entry_price
                    profit_taker = position.entry_price * self.profit_taker_multiplier
                    stop_loss = position.entry_price * self.stop_loss_multiplier

                    if close_prices[i] >= profit_taker or close_prices[i] <= stop_loss:
                        position.exit(data.loc[i, 'timestamp'], close_prices[i])
                        self.trades.append(position)
                        self.trade_data[stock]['total_profit'] += position.profit
                        self.trade_data[stock]['total_trades'] += 1
                        self.trade_data[stock]['winning_trades' if position.profit > 0 else 'losing_trades'] += 1
                        self.trade_data[stock]['balance'] += position.profit
                        self.total_balance += position.profit

                        if self.total_balance > self.highest_balance:
                            self.highest_balance = self.total_balance
                        else:
                            drawdown = self.highest_balance - self.total_balance
                            self.max_drawdown = max(self.max_drawdown, drawdown)

                        position = None

                volume_price = volumes[i] * close_prices[i]
                above_previous = close_prices[i] > highs[i - 1]

                if not position and volume_price > self.min_volume_price and above_previous:
                    position = Trade(stock, data.loc[i, 'timestamp'], close_prices[i])

            if position:
                # Close any open position at the end of the data
                position.exit(data.iloc[-1]['timestamp'], close_prices[-1])
                self.trades.append(position)
                self.trade_data[stock]['total_profit'] += position.profit
                self.trade_data[stock]['total_trades'] += 1
                self.trade_data[stock]['winning_trades' if position.profit > 0 else 'losing_trades'] += 1
                self.trade_data[stock]['balance'] += position.profit
                self.total_balance += position.profit

                if self.total_balance > self.highest_balance:
                    self.highest_balance = self.total_balance
                else:
                    drawdown = self.highest_balance - self.total_balance
                    self.max_drawdown = max(self.max_drawdown, drawdown)

    def plot_results(self):
        dates = []
        balances = []
        for trade in self.trades:
            dates.extend([trade.entry_timestamp, trade.exit_timestamp])
            balances.extend([trade.entry_price, trade.exit_price])
        dates = np.array(dates)
        balances = np.array(balances)



        print("\nCumulative Data:")
        for stock, data in self.trade_data.items():
            print(f"{stock}:")
            print(f"  Initial balance: {data['balance']}")
            print(f"  Total profit: {data['total_profit']}")
            print(f"  Total trades: {data['total_trades']}")
            print(f"  Winning trades: {data['winning_trades']}")
            print(f"  Losing trades: {data['losing_trades']}")
            print(f"  Final balance: {data['balance']}")

        print(f"\nMax Drawdown: {self.max_drawdown}")

if __name__ == "__main__":
    stock_directory = "stockDataIndiSmall"
    stock_data = load_data(stock_directory)
    backtest = Backtest(stock_data, min_volume_price=1000, profit_taker_multiplier=10, stop_loss_multiplier=1,
    initial_balance=10000)
    backtest.run()
    backtest.plot_results()