import csv
import os
import logging
import yfinance as yf
from PyQt5.QtWidgets import (QApplication, QWidget, QVBoxLayout, QTableWidget, QTableWidgetItem, QPushButton,
                              QTextEdit, QHeaderView, QMainWindow, QLabel)
from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QTimer
from PyQt5.QtGui import QColor
import pickle

def save_selected_stocks(stocks):
    print(stocks)  # add this line to check the contents of stocks
    with open('selected_stocks.pkl', 'wb') as f:
        pickle.dump(stocks, f)

class StockList:
    def __init__(self):
        self.selected_stocks = []

    def add_stock(self, ticker):
        if ticker not in self.selected_stocks:
            self.selected_stocks.append(ticker)

    def remove_stock(self, ticker):
        if ticker in self.selected_stocks:
            self.selected_stocks.remove(ticker)

    def get_stocks(self):
        return self.selected_stocks


class ButtonTableWidgetItem(QWidget):
    def __init__(self, selected_stocks, ticker, parent=None):
        super(ButtonTableWidgetItem, self).__init__(parent)
        self.selected_stocks = selected_stocks
        self.ticker = ticker

        self.button = QPushButton("Add", self)
        self.button.clicked.connect(self.toggle_button)

    def toggle_button(self):
        if self.button.text() == "Add":
            self.button.setText("Remove")
            self.selected_stocks.add_stock(self.ticker)
        else:
            self.button.setText("Add")
            self.selected_stocks.remove_stock(self.ticker)
        save_selected_stocks(self.selected_stocks.get_stocks())
        print(self.selected_stocks.get_stocks())

def load_selected_stocks():
    try:
        with open('selected_stocks.pkl', 'rb') as f:
            stocks = pickle.load(f)
    except FileNotFoundError:
        stocks = []
    return stocks


class NumericTableWidgetItem(QTableWidgetItem):
    def __init__(self, value):
        super().__init__(f"{value:.2f}")
        self.value = value

    def __lt__(self, other):
        return self.value < other.value


class LogWidget(QTextEdit):
    def __init__(self):
        super().__init__()
        self.setReadOnly(True)

    def log(self, message, color=None):
        if color:
            self.setTextColor(color)
        self.append(message)
        if color:
            self.setTextColor(Qt.black)


class LogHandler(logging.Handler):
    def __init__(self, log_widget):
        super().__init__()
        self.log_widget = log_widget

    def emit(self, record):
        msg = self.format(record)
        color = None
        if record.levelno == logging.ERROR:
            color = QColor(255, 0, 0)
        elif record.levelno == logging.INFO:
            color = QColor(0, 128, 0)
        QCoreApplication.instance().postEvent(self.log_widget, LogEvent(msg, color))


class LogEvent(QEvent):
    EVENT_TYPE = QEvent.Type(QEvent.registerEventType())

    def __init__(self, message, color=None):
        super().__init__(LogEvent.EVENT_TYPE)
        self.message = message
        self.color = color


class LoggerWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.log_widget = LogWidget()
        self.setCentralWidget(self.log_widget)
        self.setWindowTitle('Log Window')
        self.setGeometry(500, 100, 400, 300)

        # Install the custom event filter to handle LogEvent
        self.log_widget.installEventFilter(self)

    def eventFilter(self, source, event):
        if event.type() == LogEvent.EVENT_TYPE:
            self.log_widget.log(event.message, event.color)
            return True
        return super().eventFilter(source, event)

    def setup_logger(self):
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        handler = LogHandler(self.log_widget)
        handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s'))
        logger.addHandler(handler)
        return logger



def read_tickers_from_csv(files):
    tickers = []
    for file in files:
        file_path = os.path.join('Kernel', 'Stocks', file)
        with open(file_path, newline='') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                tickers.append(row[0])
    return tickers


class StockData(QWidget):
    def __init__(self, tickers, selected_stocks):
        super().__init__()
        self.tickers = tickers
        self.selected_stocks = selected_stocks

    def toggle_stock(self, ticker):
        if ticker in self.selected_stocks:
            self.selected_stocks.remove(ticker)
        else:
            self.selected_stocks.append(ticker)    
            
    def initUI(self):
        layout = QVBoxLayout()
        self.table = QTableWidget()
        layout.addWidget(self.table)
        self.setLayout(layout)
        self.setGeometry(100, 100, 1100, 600)
        self.setWindowTitle('Stock Data')

        refresh_button = QPushButton('Refresh Data', self)
        layout.addWidget(refresh_button)
        refresh_button.clicked.connect(self.load_stock_data)

        self.load_stock_data()

        self.show()

    def load_stock_data(self):
        logger = logging.getLogger()
        logger.info("Loading stock data...")
        data = yf.download(self.tickers, period="1d", group_by="ticker")

        self.table.setRowCount(len(self.tickers))
        self.table.setColumnCount(8)  # Set the column count to 8
        self.table.setHorizontalHeaderLabels(["Ticker", "Open", "High", "Low", "Close", "Volume", "Volume*Price", "Action"])
    
        timer = QTimer()
        timer.timeout.connect(QCoreApplication.processEvents)
        timer.start(0)

        for i, ticker in enumerate(self.tickers):
            try:
                stock_data = data[ticker].iloc[-1]
                self.table.setItem(i, 0, QTableWidgetItem(ticker))

                item = NumericTableWidgetItem(stock_data['Open'])
                self.table.setItem(i, 1, item)

                item = NumericTableWidgetItem(stock_data['High'])
                self.table.setItem(i, 2, item)

                item = NumericTableWidgetItem(stock_data['Low'])
                self.table.setItem(i, 3, item)

                item = NumericTableWidgetItem(stock_data['Close'])
                self.table.setItem(i, 4, item)

                item = NumericTableWidgetItem(stock_data['Volume'])
                self.table.setItem(i, 5, item)

                item = NumericTableWidgetItem(stock_data['Volume'] * stock_data['Close'])
                self.table.setItem(i, 6, item)

                button_item = ButtonTableWidgetItem(self.selected_stocks, ticker)
                self.table.setCellWidget(i, 7, button_item)

                logger.info(f"Ticker {ticker} has been successfully loaded.")
            except KeyError:
                logger.error(f"Error fetching data for {ticker}")

            logger.info(f"Completed row {i+1} of {len(self.tickers)}.")

        logger.info("Finished loading stock data.")
        self.refresh_data()  # Call refresh_data to resize columns and apply resize mode


    def refresh_data(self):
        self.table.resizeColumnsToContents()
        self.table.horizontalHeader().setSectionResizeMode(QHeaderView.Interactive)
        for i in range(self.table.columnCount()):
            self.table.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
        self.table.setSortingEnabled(True)
        self.table.sortItems(6, Qt.DescendingOrder)
        for i in range(10):
            ticker_item = self.table.item(i, 0)
            ticker = ticker_item.text()
            self.selected_stocks.add_stock(ticker)
            button_item = self.table.cellWidget(i, 7)
            button_item.toggle_button()




def main():
    app = QApplication([])
    logger_window = LoggerWindow()
    logger = logger_window.setup_logger()
    logger_window.show()

    # Load selected stocks from file
    selected_stocks = load_selected_stocks()

    stock_list = StockList()
    for ticker in selected_stocks:
        stock_list.add_stock(ticker)

    tickers = read_tickers_from_csv(['AMEX.csv'])
   # tickers = read_tickers_from_csv(['AMEX.csv','NASDAQ.csv','NYSE.csv'])
    stock_data = StockData(tickers, stock_list)
    stock_data.initUI()
    app.exec_()
    save_selected_stocks(stock_list.selected_stocks)  # Save the selected stocks to a file
    return stock_list

if __name__ == '__main__':
    selected_stocks = main()