import os
import pandas as pd
import numpy as np

def SMA(data, period, column_name):
    return data[column_name].rolling(window=period).mean()

def EMA(data, period, column_name):
    return data[column_name].ewm(span=period, adjust=False).mean()

def MACD(data, short_period, long_period, signal_period, column_name):
    short_EMA = EMA(data, short_period, column_name)
    long_EMA = EMA(data, long_period, column_name)
    data['MACD'] = short_EMA - long_EMA
    data['Signal'] = EMA(data, signal_period, 'MACD')
    return data

def RSI(data, period, column_name):
    delta = data[column_name].diff()
    gain = delta.where(delta > 0, 0)
    loss = -delta.where(delta < 0, 0)
    avg_gain = gain.rolling(window=period).mean()
    avg_loss = loss.rolling(window=period).mean()
    RS = avg_gain / avg_loss
    data['RSI'] = 100 - (100 / (1 + RS))
    return data

def BollingerBands(data, period, column_name):
    sma = SMA(data, period, column_name)
    std_dev = data[column_name].rolling(window=period).std()
    data['BB_upper'] = sma + (2 * std_dev)
    data['BB_lower'] = sma - (2 * std_dev)
    return data

def ATR(data, period):
    high_low = data['2. high'] - data['3. low']
    high_close = np.abs(data['2. high'] - data['4. close'].shift())
    low_close = np.abs(data['3. low'] - data['4. close'].shift())
    TR = pd.concat([high_low, high_close, low_close], axis=1).max(axis=1)
    data['ATR'] = TR.rolling(window=period).mean()
    return data

def OBV(data):
    volumes = data['5. volume']
    changes = data['4. close'].diff()
    obv = np.where(changes > 0, volumes, np.where(changes < 0, -volumes, 0)).cumsum()
    data['OBV'] = obv
    return data

def StochasticOscillator(data, k_period, d_period):
    high_max = data['2. high'].rolling(window=k_period).max()
    low_min = data['3. low'].rolling(window=k_period).min()
    data['%K'] = 100 * ((data['4. close'] - low_min) / (high_max - low_min))
    data['%D'] = SMA(data, d_period, '%K')
    return data

def ROC(data, period, column_name):
    data['ROC'] = ((data[column_name] - data[column_name].shift(period)) / data[column_name].shift(period)) * 100
    return data

def CCI(data, period):
    TP = (data['2. high'] + data['3. low'] + data['4. close']) / 3
    sma_tp = TP.rolling(window=period).mean()
    mean_deviation = TP.rolling(window=period).apply(lambda x: np.mean(np.abs(x - x.mean())))
    data['CCI'] = (TP - sma_tp) / (0.015 * mean_deviation)
    return data

def backtest_strategy(data):
    balance = 2000
    position = 0
    for i in range(1, len(data)):
        current_close = data.at[i, '4. close']
        previous_close = data.at[i-1, '4. close']

        if current_close > previous_close and position == 0:
            # Buy
            position = balance / current_close
            balance = 0
        elif current_close < previous_close and position != 0:
            # Sell
            balance = position * current_close
            position = 0

# Sell remaining position
    if position != 0:
        balance = position * data.at[len(data) - 1, '4. close']
        position = 0

    return balance

def main():
    output_file = 'backtest_results.csv'
    results = []
    for filename in os.listdir():
        if filename.endswith('_1min.csv'):
            stock_name = filename.split('_')[0]
            data = pd.read_csv(filename)

            # Add features to the dataset
            # data = MACD(data, 12, 26, 9, '4. close')
            #data = RSI(data, 14, '4. close')
            #data = BollingerBands(data, 20, '4. close')
            #data = ATR(data, 14)
            #data = OBV(data)
            #data = StochasticOscillator(data, 14, 3)
            #data = ROC(data, 12, '4. close')
            #data = CCI(data, 20)

            final_balance = backtest_strategy(data)
            results.append([stock_name, final_balance])

    results_df = pd.DataFrame(results, columns=['Stock', 'Final Balance'])
    results_df.to_csv(output_file, index=False)

if __name__ == '__main__':
    main()