import csv
import os
import logging
import yfinance as yf
from PyQt5.QtWidgets import (QApplication, QWidget, QVBoxLayout, QTableWidget, QTableWidgetItem, QPushButton,
                              QTextEdit, QHeaderView, QMainWindow, QLabel)
from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QTimer
from PyQt5.QtGui import QColor


class NumericTableWidgetItem(QTableWidgetItem):
    def __init__(self, value):
        super().__init__(f"{value:.2f}")
        self.value = value

    def __lt__(self, other):
        return self.value < other.value


class LogWidget(QTextEdit):
    def __init__(self):
        super().__init__()
        self.setReadOnly(True)

    def log(self, message, color=None):
        if color:
            self.setTextColor(color)
        self.append(message)
        if color:
            self.setTextColor(Qt.black)


class LogHandler(logging.Handler):
    def __init__(self, log_widget):
        super().__init__()
        self.log_widget = log_widget

    def emit(self, record):
        msg = self.format(record)
        color = None
        if record.levelno == logging.ERROR:
            color = QColor(255, 0, 0)
        elif record.levelno == logging.INFO:
            color = QColor(0, 128, 0)
        QCoreApplication.instance().postEvent(self.log_widget, LogEvent(msg, color))


class LogEvent(QEvent):
    EVENT_TYPE = QEvent.Type(QEvent.registerEventType())

    def __init__(self, message, color=None):
        super().__init__(LogEvent.EVENT_TYPE)
        self.message = message
        self.color = color


class LoggerWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.log_widget = LogWidget()
        self.setCentralWidget(self.log_widget)
        self.setWindowTitle('Log Window')
        self.setGeometry(500, 100, 400, 300)

        # Install the custom event filter to handle LogEvent
        self.log_widget.installEventFilter(self)

    def eventFilter(self, source, event):
        if event.type() == LogEvent.EVENT_TYPE:
            self.log_widget.log(event.message, event.color)
            return True
        return super().eventFilter(source, event)

    def setup_logger(self):
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        handler = LogHandler(self.log_widget)
        handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s'))
        logger.addHandler(handler)
        return logger



def read_tickers_from_csv(files):
    tickers = []
    for file in files:
        file_path = os.path.join('Kernel', 'Stocks', file)
        with open(file_path, newline='') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                tickers.append(row[0])
    return tickers


class StockData(QWidget):
    def __init__(self, tickers):
        super().__init__()
        self.tickers = tickers
        self.selected_rows = []

    def initUI(self):
        layout = QVBoxLayout()
        self.table = QTableWidget()
        layout.addWidget
        self.setLayout(layout)
        self.setGeometry(100, 100, 1100, 600)
        self.setWindowTitle('Stock Data')

        refresh_button = QPushButton('Refresh Data', self)
        layout.addWidget(self.table)
        refresh_button.clicked.connect(self.load_stock_data)

        self.load_stock_data()

        self.show()

    def load_stock_data(self):
        logger = logging.getLogger()
        logger.info("Loading stock data...")
        data = yf.download(self.tickers, period="1d", group_by="ticker")

        self.table.setRowCount(len(self.tickers))
        self.table.setColumnCount(8)  # Set the column count to 8
        self.table.setHorizontalHeaderLabels(["Ticker", "Open", "High", "Low", "Close", "Volume", "Volume*Price", "Action"])

        timer = QTimer()
        timer.timeout.connect(QCoreApplication.processEvents)
        timer.start(0)

        for i, ticker in enumerate(self.tickers):
            try:
                stock_data = data[ticker].iloc[-1]
                self.table.setItem(i, 0, QTableWidgetItem(ticker))

                item = NumericTableWidgetItem(stock_data['Open'])
                self.table.setItem(i, 1, item)

                item = NumericTableWidgetItem(stock_data['High'])
                self.table.setItem(i, 2, item)

                item = NumericTableWidgetItem(stock_data['Low'])
                self.table.setItem(i, 3, item)

                item = NumericTableWidgetItem(stock_data['Close'])
                self.table.setItem(i, 4, item)

                item = NumericTableWidgetItem(stock_data['Volume'])
                self.table.setItem(i, 5, item)

                item = NumericTableWidgetItem(stock_data['Volume'] * stock_data['Close'])
                self.table.setItem(i, 6, item)

                add_button = QPushButton('Add', self)
                add_button.clicked.connect(lambda _, i=i: self.on_button_clicked(i))
                self.table.setCellWidget(i, 7, add_button)

                logger.info(f"Ticker {ticker} has been successfully loaded.")
            except KeyError:
                logger.error(f"Error fetching data for {ticker}")

            logger.info(f"Completed row {i+1} of {len(self.tickers)}.")

        # Sort the rows based on Volume*Price and select the top 10


        logger.info("Finished loading stock data.")
        self.refresh_data()  # Call refresh_data to resize columns and apply resize mode


    def refresh_data(self):
        self.table.resizeColumnsToContents()
        self.table.horizontalHeader().setSectionResizeMode(QHeaderView.Interactive)
        for i in range(self.table.columnCount()):
            self.table.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
        self.table.setSortingEnabled(True)

    def on_button_clicked(self, row):
        ticker = self.table.item(row, 0).text()
        if ticker in self.selected_rows:
            self.selected_rows.remove(ticker)
            self.table.cellWidget(row, 7).setText('Add')
        else:
            self.selected_rows.append(ticker)
            self.table.cellWidget(row, 7).setText('Remove')

    def get_selected_stocks(self):
        data = yf.download(self.selected_rows, period="1d", group_by="ticker")
        selected_stocks = []
        for ticker in self.selected_rows:
            try:
                stock_data = data[ticker].iloc[-1]
                selected_stocks.append([ticker, stock_data['Open'], stock_data['High'], stock_data['Low'],
                                    stock_data['Close'], stock_data['Volume'], stock_data['Volume']*stock_data['Close']])
            except KeyError:
                logger.error(f"Error fetching data for {ticker}")

def main():
    app = QApplication([])
    logger_window = LoggerWindow()
    logger = logger_window.setup_logger()
    logger_window.show()
    tickers = read_tickers_from_csv(['AMEX.csv'])
   # tickers = read_tickers_from_csv(['AMEX.csv','NASDAQ.csv','NYSE.csv'])
    stock_data = StockData(tickers)
    stock_data.initUI()

    def on_get_selected_stocks_clicked():
        selected_stocks = stock_data.get_selected_stocks()
        for stock in selected_stocks:
            logger.info(f"{stock[0]}: Open={stock[1]}, High={stock[2]}, Low={stock[3]}, Close={stock[4]}, Volume={stock[5]}, Volume*Price={stock[6]}")

    get_selected_stocks_button = QPushButton('Get Selected Stocks', logger_window)
    get_selected_stocks_button.clicked.connect(on_get_selected_stocks_clicked)
    toolbar = QToolBar(logger_window)
    toolbar.addWidget(get_selected_stocks_button)
    logger_window.addToolBar(Qt.BottomToolBarArea, toolbar)

    app.exec_()


if __name__ == '__main__':
    main()