import os
import glob
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.stats import ttest_ind



def analyze_stock(stock_df):
    # Calculate the daily returns
    try:
        stock_df['daily_return'] = stock_df['close'].pct_change()
    except ZeroDivisionError:
        print("Error: Cannot calculate percentage change. Possible division by zero.")
        return None

    # Calculate the log returns
    try:
        stock_df['log_return'] = np.log(stock_df['close']).diff()
    except ZeroDivisionError:
        print("Error: Cannot calculate log returns. Possible division by zero.")
        return None

    # Calculate the moving averages
    try:
        stock_df['sma_10'] = stock_df['close'].rolling(window=10).mean()
        stock_df['sma_20'] = stock_df['close'].rolling(window=20).mean()
        stock_df['ema_10'] = stock_df['close'].ewm(span=10).mean()
        stock_df['ema_20'] = stock_df['close'].ewm(span=20).mean()
    except ZeroDivisionError:
        print("Error: Cannot calculate moving averages. Possible division by zero.")
        return None

    # Calculate the Bollinger Bands
    try:
        rolling_mean = stock_df['close'].rolling(window=20).mean()
        rolling_std = stock_df['close'].rolling(window=20).std()
        stock_df['bb_upper'] = rolling_mean + 2 * rolling_std
        stock_df['bb_lower'] = rolling_mean - 2 * rolling_std
    except ZeroDivisionError:
        print("Error: Cannot calculate Bollinger Bands. Possible division by zero.")
        return None

    # Calculate the Relative Strength Index (RSI)
    try:
        delta = stock_df['close'].diff()
        gain = delta.where(delta > 0, 0)
        loss = -delta.where(delta < 0, 0)
        avg_gain = gain.rolling(window=14).mean()
        avg_loss = loss.rolling(window=14).mean()
        rs = avg_gain / avg_loss
        stock_df['rsi'] = 100 - (100 / (1 + rs))
    except ZeroDivisionError:
        print("Error: Cannot calculate RSI. Possible division by zero.")
        return None

    # Calculate the Moving Average Convergence Divergence (MACD)
    try:
        exp1 = stock_df['close'].ewm(span=12, adjust=False).mean()
        exp2 = stock_df['close'].ewm(span=26, adjust=False).mean()
        stock_df['macd'] = exp1 - exp2
        stock_df['signal'] = stock_df['macd'].ewm(span=9, adjust=False).mean()
    except ZeroDivisionError:
        print("Error: Cannot calculate MACD. Possible division by zero.")
        return None

    # Calculate the On-Balance Volume (OBV)
    try:
        stock_df['obv'] = np.where(stock_df['daily_return'] > 0, stock_df['volume'], -stock_df['volume']).cumsum()
    except ZeroDivisionError:
        print("Error: Cannot calculate OBV. Possible division by zero.")
        return None

# Perform a t-test on the daily returns to test for significance
    try:
        mean_return = stock_df['daily_return'].mean()
        t_stat, p_value = ttest_ind(stock_df['daily_return'], np.zeros_like(stock_df['daily_return']))
        is_significant = p_value < 0.05
    except ZeroDivisionError:
        print("Error: Cannot perform t-test. Possible division by zero.")
        return None


    # Return a dictionary of results
    results = {
        'mean_return': mean_return,
        't_stat': t_stat,
        'p_value': p_value,
        'is_significant': is_significant
    }

    return results

def analyze_stocks(input_folder, output_folder):
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    file_paths = glob.glob(f"{input_folder}/*_5min_normalized_indicators.csv")

    results_list = []
    performance = pd.DataFrame(columns=['Indicator', 'Sharpe Ratio', 'Total Return', 'Annualized Return', 'Max Drawdown'])


    for file_path in file_paths:
        stock_name = os.path.basename(file_path).split("_")[0]

        df = pd.read_csv(file_path)
        df['timestamp'] = pd.to_datetime(df['timestamp'])
        df.set_index('timestamp', inplace=True)


        # Filter rows where the product of volume and price is greater than 10,000
        df = df[df['volume'] * df['close'] > 10000]

        if len(df) > 0:
            stock_results = analyze_stock(df)
            # ...
        else:
            print(f"Not enough data for stock {stock_name} after filtering. Skipping...")
            continue

        if stock_results is None:
            print(f"Analysis for stock {stock_name} failed. Skipping...")
            continue

        stock_results['stock_name'] = stock_name
        results_list.append(stock_results)

        # Compute the Sharpe Ratio for each indicator
        performance = pd.DataFrame(columns=['Indicator', 'Sharpe Ratio', 'Total Return', 'Annualized Return', 'Max Drawdown'])

        for indicator in ['sma_10', 'sma_20', 'ema_10', 'ema_20', 'macd']:
            try:
                indicator_returns = df['close'] / df[indicator] - 1
                indicator_returns.fillna(0, inplace=True)
                indicator_sharpe_ratio = indicator_returns.mean() / indicator_returns.std() * np.sqrt(252)
                indicator_total_return = (indicator_returns + 1).cumprod().iloc[-1] - 1
                indicator_annualized_return = (1 + indicator_total_return) ** (252 / len(df)) - 1
                indicator_max_drawdown = (indicator_returns + 1).cumprod().div((indicator_returns + 1).cumprod().cummax()).sub(1).min()

                performance = performance.append({'Indicator': indicator.upper(), 'Sharpe Ratio': indicator_sharpe_ratio, 
                                                  'Total Return': indicator_total_return, 
                                                  'Annualized Return': indicator_annualized_return,
                                                  'Max Drawdown': indicator_max_drawdown}, ignore_index=True)
            except ZeroDivisionError:
                print(f"Error: Cannot calculate performance for {indicator.upper()}. Possible division by zero.")

        # Compute the Sharpe Ratio for the Bollinger Bands strategy
        try:
            rolling_mean = df['close'].rolling(window=20).mean()
            rolling_std = df['close'].rolling(window=20).std()
            df['bb_upper'] = rolling_mean + 2 * rolling_std
            df['bb_lower'] = rolling_mean - 2 * rolling_std
            
            bb_returns = np.where(df['close'] > df['bb_upper'], -1, np.nan)
            bb_returns = np.where(df['close'] < df['bb_lower'], 1, bb_returns)
            bb_returns = pd.Series(bb_returns).fillna(0)
            bb_sharpe_ratio = bb_returns.mean() / bb_returns.std() * np.sqrt(252)
            bb_total_return = (bb_returns + 1).cumprod().iloc[-1] - 1
            bb_annualized_return = (1 + bb_total_return) ** (252 / len(df)) - 1
            bb_max_drawdown = (bb_returns + 1).cumprod().div((bb_returns + 1).cumprod().cummax()).sub(1).min()

            performance = performance.append({'Indicator': 'Bollinger Bands', 'Sharpe Ratio': bb_sharpe_ratio, 
                                              'Total Return': bb_total_return, 
                                              'Annualized Return': bb_annualized_return,
                                              'Max Drawdown': bb_max_drawdown}, ignore_index=True)
        except ZeroDivisionError:
            print("Error: Cannot calculate performance for Bollinger Bands. Possible division by zero.")


        # Compute the Sharpe Ratio for the RSI strategy
        rsi_returns = np.where(df['rsi'] > 70, -1, np.nan)
        rsi_returns = np.where(df['rsi'] < 30, 1, rsi_returns)
        rsi_returns = pd.Series(rsi_returns).fillna(0)
        rsi_sharpe_ratio = rsi_returns.mean() / rsi_returns.std() * np.sqrt(252)
        rsi_total_return = (rsi_returns + 1).cumprod().iloc[-1] - 1
        rsi_annualized_return = (1 + rsi_total_return) ** (252 / len(df)) - 1
        rsi_max_drawdown = (rsi_returns + 1).cumprod().div((rsi_returns + 1).cumprod().cummax()).sub(1).min()

        performance = performance.append({'Indicator': 'RSI', 'Sharpe Ratio': rsi_sharpe_ratio, 
                                        'Total Return': rsi_total_return, 'Annualized Return': rsi_annualized_return,
                                        'Max Drawdown': rsi_max_drawdown}, ignore_index=True)

        # Compute the Sharpe Ratio for the MACD strategy
        macd_returns = np.where(df['macd'] > df['signal'], -1, np.nan)
        macd_returns = np.where(df['macd'] < df['signal'], 1, macd_returns)
        macd_returns = pd.Series(macd_returns).fillna(0)

        macd_sharpe_ratio = macd_returns.mean() / macd_returns.std() * np.sqrt(252)
        macd_total_return = (macd_returns + 1).cumprod().iloc[-1] - 1
        macd_annualized_return = (1 + macd_total_return) ** (252 / len(df)) - 1
        macd_max_drawdown = (macd_returns + 1).cumprod().div((macd_returns + 1).cumprod().cummax()).sub(1).min()

        performance = performance.append({'Indicator': 'MACD', 'Sharpe Ratio': macd_sharpe_ratio,
                                        'Total Return': macd_total_return, 'Annualized Return': macd_annualized_return,
                                        'Max Drawdown': macd_max_drawdown}, ignore_index=True)

    # Sort the performance DataFrame by Sharpe Ratio
    performance.sort_values('Sharpe Ratio', ascending=False, inplace=True)

    # Save the performance results to a CSV file
    performance.to_csv(os.path.join(output_folder, 'indicator_performance.csv'), index=False)

    # Create a plot of the total returns for each indicator
    plt.figure(figsize=(10, 6))
    plt.bar(performance['Indicator'], performance['Total Return'])
    plt.xticks(rotation=45, ha='right')
    plt.xlabel('Indicator')
    plt.ylabel('Total Return')
    plt.title('Total Returns by Indicator')
    plt.tight_layout()
    plt.savefig(os.path.join(output_folder, 'total_returns.png'))

    # Create a plot of the Sharpe Ratio for each indicator
    plt.figure(figsize=(10, 6))
    plt.bar(performance['Indicator'], performance['Sharpe Ratio'])
    plt.xticks(rotation=45, ha='right')
    plt.xlabel('Indicator')
    plt.ylabel('Sharpe Ratio')
    plt.title('Sharpe Ratio by Indicator')
    plt.tight_layout()
    plt.savefig(os.path.join(output_folder, 'sharpe_ratio.png'))

input_folder = "stockDataIndiSmall"
output_folder = "stockResults2"

analyze_stocks(input_folder, output_folder)